<?php
/**
* @author zerosix.com
* @copyright zerosix.com
* @license zerosix.com
**/

class PhoneLocale {
    public function __construct($name, $dial_in_code, $mobile_pattern='\d+$', $dial_out_code='00', $trunc_code='0') {
        $this->name = $name;
        $this->dial_in_code = $dial_in_code;
        $this->mobile_pattern = $mobile_pattern;
        $this->dial_out_code = $dial_out_code;
        $this->trunc_code = $trunc_code;
        $this->international_prefix = '+'.$dial_in_code;
    }
}

global $phone_locales;
$phone_locales = array(
    'albania'=> new PhoneLocale('albania', '355', '/[6]\d{8}$/'),
    'algeria'=> new PhoneLocale('algeria', '213', '/[567]\d{8}$/'),
    'andorra'=> new PhoneLocale('andorra', '376', '/[346]\d{5}$/'),
    'argentina'=> new PhoneLocale('argentina', '54', '/[9]\d{10}$/'),
    'australia'=> new PhoneLocale('australia', '61', '/[4]\d{8}$/'),
    'austria'=> new PhoneLocale('austria', '43', '/[6]\d{9}$/'),
    'azerbaijan'=> new PhoneLocale('azerbaijan', '994', '/[4567]\d{8}$/'),
    'bahamas'=> new PhoneLocale('bahamas', '1', '/\d{10}$/'),
    'bahrain'=> new PhoneLocale('bahrain', '973', '/[36]\d{7}$/'),
    'belgium'=> new PhoneLocale('belgium', '32', '/[4]\d{8}$/'),
    'benin'=> new PhoneLocale('benin', '229', '/^[69]\d{7}$/'),
    'bosnia'=> new PhoneLocale('bosnia', '387', '/[6](?:(\d{7})|(\d{8}))$/'),
    'brazil'=> new PhoneLocale('brazil', '55', '/^\d{2}[9]\d{8}$/'),
    'bulgaria'=> new PhoneLocale('bulgaria', '359', '/(?:([4][3]\d{6})|([8][7-9]\d{7})|([9][89]\d{7}))$/'),
    'canada'=> new PhoneLocale('canada', '1', '/\d{10}$/'),
    'cayman'=> new PhoneLocale('cayman', '1', '/\d{10}$/'),
    'chile'=> new PhoneLocale('chile', '56', '/[9]\d{8}$/'),
    'china'=> new PhoneLocale('china', '86', '/[1][3-9]\d{9}$/'),
    'croatia'=> new PhoneLocale('croatia', '385', '/[9]\d{8}$/'),
    'czechrepublic'=> new PhoneLocale('czechrepublic', '420', '/[67][02379]\d{7}$/'),
    'cyprus'=> new PhoneLocale('cyprus', '357', '/[9][5679]\d{6}$/'),
    'denmark'=> new PhoneLocale('denmark', '45', '/\d{8}$/'),
    'emirates'=> new PhoneLocale('emirates', '971', '/[5]\d{8}$/'),
    'england'=> new PhoneLocale('england', '44', '/^[7]\d{9}$/'),
    'estonia'=> new PhoneLocale('estonia', '372', '/(?:([5](\d{6}|\d{7})$)|([8]\d{7})$)/'),
    'finland'=> new PhoneLocale('finland', '358', '/(?:([4]\d{9}$)|([5][0]\d{8}$))/'),
    'france'=> new PhoneLocale('france', '33', '/[67]\d{8}$/'),
    'french_polynesia'=> new PhoneLocale('french_polynesia', '689', '/[8]\d{7}$/'),
    'french_guyana'=> new PhoneLocale('french_guyana', '594', '/[6][9][4]\d{6}$/'),
    'germany'=> new PhoneLocale('germany', '49', '/(?:([1][67](\d{8}|\d{9})$)|([1][5]\d{9})$)/'),
    'greece'=> new PhoneLocale('greece', '30', '/[6]\d{9}/'),
    'guadeloupe'=> new PhoneLocale('guadeloupe', '590', '/[67]\d{8}$/'),
    'hongkong'=> new PhoneLocale('hongkong', '852', '/[4-9]\d{8}$/'),
    'hungary'=> new PhoneLocale('hungary', '36', '/(?:([257][0]\d{7}$)|([3][01]\d{7}$))/'),
    'ireland'=> new PhoneLocale('ireland', '353', '/^[8][2-9]\d{7}$/'),
    'israel'=> new PhoneLocale('israel', '972', '/[5]\d{8}$/'),
    'italy'=> new PhoneLocale('italy', '39', '/[3]\d{9}$/'),
    'ivory_coast'=> new PhoneLocale('ivory_coast', '225', '/[04567]\d{7}$/'),
    'japan'=> new PhoneLocale('japan', '81', '/[789][0]\d{8}$/'),
    'kuwait'=> new PhoneLocale('kuwait', '965', '/[569]\d{8}$/'),
    'latvia'=> new PhoneLocale('latvia', '371', '/[2]\d{7}$/'),
    'lebanon'=> new PhoneLocale('lebanon', '961', '/(?:([3]\d{6}$)|([78]\d{7}$))/'),
    'liechtenstein'=> new PhoneLocale('liechtenstein', '423', '/[7]\d{6}$/'),
    'lithuania'=> new PhoneLocale('lithuania', '370', '/[6]\d{7}$/'),
    'luxemburg'=> new PhoneLocale('luxemburg', '352', '/[67]\d{8}$/', $trunc_code=''),
    'macedonia'=> new PhoneLocale('macedonia', '389', '/[7]\d{7}$/'),
    'madagascar'=> new PhoneLocale('madagascar', '261', '/[3]\d{8}$/'),
    'malta'=> new PhoneLocale('malta', '356', '/[79]\d{7}/'),
    'martinique'=> new PhoneLocale('martinique', '596', '/[6][9][6]\d{6}$/'),
    'mayotte'=> new PhoneLocale('mayotte', '262', '/[6]\d{8}$/'),
    'moldova'=> new PhoneLocale('moldova', '373', '/[67]\d{7}$/'),
    'monaco'=> new PhoneLocale('monaco', '377', '/(?:([34]\d{7}$)|([6]\d{8}$)/', $trunc_code=''),
    'montenegro'=> new PhoneLocale('montenegro', '382', '/[6]\d{7}$/'),
    'morocco'=> new PhoneLocale('morocco', '212', '/[67]\d{8}$/'),
    'netherlands'=> new PhoneLocale('netherlands', '31', '/^[6]\d{8}$/'),
    'netherlands_antilles'=> new PhoneLocale('netherlands_antilles', '599', '/^\d{7}$/'),
    'new_caledonia'=> new PhoneLocale('new_caledonia', '687', '/[5789]\d{5}$/', $trunc_code=''),
    'norway'=> new PhoneLocale('norway', '47', '/[49]\d{7}$/'),
    'poland'=> new PhoneLocale('poland', '48', '/[5678][0123689]\d{7}$/'),
    'portugal'=> new PhoneLocale('portugal', '351', '/[9]\d{8}$/'),
    'qatar'=> new PhoneLocale('qatar', '974', '/[3567][3567]\d{6}$/'),
    'reunion'=> new PhoneLocale('reunion', '262', '/[6]\d{8}$/'),
    'romania'=> new PhoneLocale('romania', '40', '/[7]\d{8}$/'),
    'russia'=> new PhoneLocale('russia', '7', '/[9]\d{9}$/'),
    'saudiarabia'=> new PhoneLocale('saudiarabia', '966', '/[5]\d{8}$/'),
    'senegal'=> new PhoneLocale('senegal', '221', '/[7]\d{8}$/'),
    'serbia'=> new PhoneLocale('serbia', '381', '/[6](\d{7}|\d{8})$/'),
    'seychelles'=> new PhoneLocale('seychelles', '248', '/[2]\d{6}$/'),
    'singapore'=> new PhoneLocale('singapore', '65', '/[89]\d{7}$/'),
    'slovakia'=> new PhoneLocale('slovakia', '421', '/[9]\d{8}$/'),
    'slovenia'=> new PhoneLocale('slovenia', '386', '/[34567]\d{7}$/'),
    'south_africa'=> new PhoneLocale('south_africa', '27', '/[678]\d{8}$/'),
    'spain'=> new PhoneLocale('spain', '34', '/^[67]\d{8}$/', $trunc_code=''),
    'sweden'=> new PhoneLocale('sweden', '46', '/[7]\d{8}$/'),
    'swiss'=> new PhoneLocale('swiss', '41', '/[78]\d{8}$/'),
    'thailand'=> new PhoneLocale('thailand', '66', '/[89]\d{8}$/'),
    'togo'=> new PhoneLocale('togo', '228', '/^[9]\d{7}$/'),
    'tunisia'=> new PhoneLocale('tunisia', '216', '/[2459]\d{7}$/'),
    'turkey'=> new PhoneLocale('turkey', '90', '/[5]\d{9}$/'),
    'ukraine'=> new PhoneLocale('ukraine', '380', '/(?:([3][9])|([5][0])|([6][3678])|([9][1-9]))\d{7}$/'),
    'usa'=> new PhoneLocale('usa', '1', '/\d{10}$/'),
);

function get_phone_locale() {
    global $phone_locales;
    $used_phone_locale = null;
    $dial_in_code = Configuration::get('ZEROSIX_PHONE_CODE');
    foreach ($phone_locales as $country_name => $phone_locale) {
        if($phone_locale->dial_in_code == $dial_in_code) {
            $used_phone_locale = $phone_locale;
            break;
        }
    }
    return $used_phone_locale;
}

function startsWith( $number, $search ) {
    $length = strlen( $search );
    return substr( $number, 0, $length ) === $search;
}

function zerosix_is_valid_mobile_number($number) {
    global $phone_locales;
    if ($number){
        foreach ($phone_locales as $country_name => $phone_locale) {
            if(startsWith($number, $phone_locale->international_prefix)) {
                return preg_match($phone_locale->mobile_pattern, substr($number, strlen($phone_locale->international_prefix)));
            }
        }
    }
    return False;
}

function zerosix_localize_mobile_number($number){
    $used_phone_locale = get_phone_locale();
    if($used_phone_locale) {
        if($number && startsWith($number, $used_phone_locale->international_prefix)) {
            return ''.$used_phone_locale->trunc_code.''.substr($number, strlen($used_phone_locale->international_prefix));
        }
    }
    return $number;
}

function zerosix_normalize_mobile_number($number) {
    $phone_locale = get_phone_locale();
    $number = str_replace(array('-', ' ', '.'), '', $number);
    if(!startsWith($number, '+') && preg_match('/\d+$/', $number)) {
        if(startsWith($number, $phone_locale->dial_out_code)) {
            $number = '+'.substr($number, strlen($phone_locale->dial_out_code));
        }
        elseif(startsWith($number, '00'.$phone_locale->dial_in_code)) {
            $number = '+'.$phone_locale->dial_in_code.''.substr($number, strlen($phone_locale->trunc_code)+2);
        }
        elseif(startsWith($number, $phone_locale->trunc_code)) {
            $number = '+'.$phone_locale->dial_in_code.''.substr($number, strlen($phone_locale->trunc_code));
        }
        elseif(strlen($number) && $phone_locale->trunc_code == '0') {
            $number = '+'.$phone_locale->dial_in_code.''.$number;
        }
    }
    return $number;
}
