<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

define("URL_API", "https://api.zerosix.com");

define("ZS_MODULE_VERSION", "1.7.6");

include_once dirname(__FILE__).'/phone_locale.php';

class Zerosix extends Module
{
    protected $config_form = false;
    public $is_ps17 = false;
    public $is_ps1765 = false;
    public $is_pos = false;
    public $pos = array('kerawen');

    public function __construct()
    {
        if(version_compare(_PS_VERSION_,'1.7.0') >= 0)
        {
          $this->is_ps17 = true;
          if(version_compare(_PS_VERSION_,'1.7.6.5') >= 0) {
            $this->is_ps1765 = true;
          }
        }
        $this->name = 'zerosix';
        $this->tab = 'advertising_marketing';
        $this->version = ZS_MODULE_VERSION;
        $this->author = 'ZEROSIX';
        $this->need_instance = 0;

        $this->confirmUninstall = $this->l('If you delete this module, all of your ZEROSIX activated cards will be deleted from Prestashop, do you want to continue ?');


        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('ZEROSIX');
        $this->description = $this->l('Zerosix loyalty program');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);

        foreach ($this->pos as $pos_name) {
          if (Module::isInstalled($pos_name) && Module::isEnabled($pos_name)) {
            $this->is_pos = true;
          }
        }
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install() {
      return parent::install() &&
          $this->registerHook('header') &&
          $this->installDB() &&
          Configuration::updateValue('ZEROSIX_CANCELATION_MAPPING', Tools::jsonEncode([6, 7]) ) && // cancel, refund
          Configuration::updateValue('ZEROSIX_VALIDATION_MAPPING', Tools::jsonEncode([2]) ) && // payment
          Configuration::updateValue('_ZEROSIX_TOKEN_', $this->generateRandomString() ) &&
          Configuration::updateValue('ZEROSIX_PHONE_CODE', '+33') &&
          $this->registerHook('actionObjectCustomerAddAfter') &&
          $this->registerHook('actionCustomerAccountAdd') &&
          $this->registerHook('actionObjectCustomerDeleteAfter') &&
          $this->registerHook('actionObjectCartRuleDeleteAfter') &&
          $this->registerHook('actionValidateOrder') &&
          $this->registerHook('actionOrderStatusUpdate') &&
          $this->registerHook('displayCustomerForm') &&
          $this->registerHook('displayCustomerAccountForm') &&
          $this->registerHook('displayShoppingCartFooter') &&
          $this->registerHook('dashboardZoneTwo') &&
          $this->registerHook('displayPaymentTop') &&
          $this->registerHook('displayAdminCustomers') &&
          $this->registerHook('displayMyAccountBlock') &&
          $this->registerHook('displayHeader') &&
          $this->registerHook('displayReassurance') && // display cashback 1.7
          $this->registerHook('displayCustomerAccount');
    }

    /*
        Return string
        generate random token
    */
    public function generateRandomString($length = 10) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, Tools::strlen($characters) - 1)];
        }
        return md5($randomString);
    }

    public function installDB()
    {
      $sql = "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."zerosix_map_user` (
          `id_customer_presta` int(11) NOT NULL,
          `id_user_zs` int(11) NOT NULL,
          `id_card_zs` int(11) NOT NULL,
          `phone_num` TEXT NOT NULL,
          `id_shop` int(11) NOT NULL
        ) DEFAULT CHARSET=utf8;

        CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."zerosix_map_visits` (
            `id_order_ps` int(11) NOT NULL,
            `id_visit_zs` int(11) NOT NULL
          ) DEFAULT CHARSET=utf8;


        CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."zerosix_map_vouchers` (
          `id_discount_ps` int(11) NOT NULL,
          `id_user_ps` int(11) NOT NULL,
          `id_voucher_zs` varchar(30) NOT NULL,
          `type` varchar(30) NOT NULL
        )  DEFAULT CHARSET=utf8;


        CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."zerosix_waiting_otp` (
          `id_user` int(11) NOT NULL,
          `number` text NOT NULL,
          `zs_card` text NOT NULL
        )  DEFAULT CHARSET=utf8;";

        Db::getInstance()->execute($sql);

        $sql  = "ALTER TABLE `"._DB_PREFIX_."zerosix_map_user` ADD UNIQUE (`id_customer_presta`, `id_user_zs`, `id_card_zs`, `id_shop`);";
        Db::getInstance()->execute($sql);

        $sql  = "ALTER TABLE `"._DB_PREFIX_."zerosix_map_visits` ADD UNIQUE KEY `id_order_ps` (`id_order_ps`,`id_visit_zs`);";

        Db::getInstance()->execute($sql);
        
        $sql  = "ALTER TABLE `"._DB_PREFIX_."zerosix_map_vouchers` ADD UNIQUE (`id_discount_ps`, `id_user_ps`, `id_voucher_zs`);";
        
        Db::getInstance()->execute($sql);

        $sql = "ALTER TABLE `"._DB_PREFIX_."zerosix_waiting_otp` ADD PRIMARY KEY (`id_user`)";
        
        return Db::getInstance()->execute($sql);

    }

    public function removeDB()
    {
      $sql = "DROP TABLE IF EXISTS "._DB_PREFIX_."zerosix_map_user";
      Db::getInstance()->execute($sql);

      $sql = "DROP TABLE IF EXISTS "._DB_PREFIX_."zerosix_map_visits";
      Db::getInstance()->execute($sql);
      
      $sql = "DROP TABLE IF EXISTS "._DB_PREFIX_."zerosix_map_vouchers";
      Db::getInstance()->execute($sql);
      
      
      $sql = "DROP TABLE IF EXISTS "._DB_PREFIX_."zerosix_waiting_otp";
      return Db::getInstance()->execute($sql);
    }

    public function uninstall()
    {
        Configuration::deleteByName('ZEROSIX_LIVE_MODE');

        return parent::uninstall() && $this->removeDB();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        $output = "";
        if (((bool)Tools::isSubmit('submitZerosixModule')) == true) {
            $postProcess = $this->postProcess();
            if($postProcess['success'] == true)
              $output .= $this->displayConfirmation($this->l('Settings saved'));
            else
              $output .= $postProcess['errors'];
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output .= $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    public function getIdFromZSApiUrl($url) {
      $array = explode('/', rtrim($url, '/'));
      return array_pop($array);
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitZerosixModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {

        $states = OrderState::getOrderStates($this->context->language->id);

        $statesChoices = array();
        foreach($states as $state) {
          $statesChoices[] = array(
            'id_option' => $state["id_order_state"],
            'name' => $state["name"]
          );
        }

        $input = array(
          // api field
          // Dashboard url
          array(
              'col' => 3,
              'type' => 'text',
              'prefix' => '<i class="icon icon-link"></i>',
              'desc' => $this->l('Paste the url of your dashboard'),
              'name' => 'ZEROSIX_URL_DASHBOARD',
              'label' => $this->l('Dashboard uri'),
          ),
          // loyalty program uri
          array(
              'col' => 3,
              'type' => 'text',
              'prefix' => '<i class="icon icon-link"></i>',
              'desc' => $this->l('Paste the url of your loyalty program description page'),
              'name' => 'ZEROSIX_URL_LOYALTY',
              'label' => $this->l('Loyalty program URI'),
          ),
          // term and cond uri program
          array(
              'col' => 3,
              'type' => 'text',
              'prefix' => '<i class="icon icon-link"></i>',
              'desc' => $this->l('Paste the url of your terms and conditions page'),
              'name' => 'ZEROSIX_URL_TERMS',
              'label' => $this->l('Terms and conditions page URI'),
          ),
          // Indicatif téléphonique
          array(
            'col' => 3,
            'type' => 'text',
            'desc' => $this->l('Fill your country phone code'),
            'name' => 'ZEROSIX_PHONE_CODE',
            'label' => $this->l('Phone code'),
        ),
        // Sponsorship
          array(
            'type' => 'switch',
            'label' => $this->l('Sponsorship enabled ?'),
            'name' => 'ZEROSIX_SPONSORSHIP',
            'is_bool' => true,
            'values' => array(
              array(
                'id' => 'active_on',
                'value' => 1,
                'label' => $this->l('Yes')
              ),
              array(
                'id' => 'active_off',
                'value' => 0,
                'label' => $this->l('No')
              )
            ),
          ),
          // Sponsorship page
          array(
            'col' => 3,
            'type' => 'text',
            'prefix' => '<i class="icon icon-link"></i>',
            'desc' => $this->l('Paste the url of your sponsorship feature page'),
            'name' => 'ZEROSIX_URL_SPONSORSHIP',
            'label' => $this->l('Sponsorship page uri'),
        ),
          //validate order state choice
          array(
            'type' => 'checkbox',                             
            'label' => $this->l('Validation'),         
            'desc' => $this->l('Choose a validation method'),  
            'name' => 'ZEROSIX_VALIDATION_MAPPING',                     
            'required' => true,                              
            'values' => array(
              'query' => $statesChoices,                           
              'id' => 'id_option',                           
              'name' => 'name'                               
            )
          ),
          // cancel states choices
          array(
            'type' => 'checkbox',                             
            'label' => $this->l('Cancelation'),         
            'desc' => $this->l('Choose a cancelation method'),  
            'name' => 'ZEROSIX_CANCELATION_MAPPING',                     
            'required' => false,                              
            'values' => array(
              'query' => $statesChoices,                           
              'id' => 'id_option',                           
              'name' => 'name'                               
            )
          ),
        );

        $shops = Shop::getShops();
        foreach(array_reverse($shops) as $shop) {
          if($this->is_pos) {
            foreach($this->pos as $pos_name)
            $field_name = 'ZEROSIX_API_KEY_'.$pos_name.'_'.$shop['id_shop'];
            array_unshift($input, array(
              'col' => 3,
              'type' => 'text',
              'prefix' => '<i class="icon icon-key"></i> '.$shop['name'].' '.$pos_name,
              'desc' => $this->l('Paste your API KEY'),
              'name' => $field_name,
              'label' => $this->l('API KEY'),
            ));
          }
          $field_name = 'ZEROSIX_API_KEY_'.$shop['id_shop'];
          array_unshift($input, array(
            'col' => 3,
            'type' => 'text',
            'prefix' => '<i class="icon icon-key"></i> '.$shop['name'],
            'desc' => $this->l('Paste your API KEY'),
            'name' => $field_name,
            'label' => $this->l('API KEY'),
          ));
        }

        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => $input,
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    public function getValidationMapping() {
      $validation_mapping = Tools::jsonDecode(Configuration::get('ZEROSIX_VALIDATION_MAPPING'));
      if (!is_array($validation_mapping)) {
        $validation_mapping = [];
      }
      return $validation_mapping;
    }

    public function getCancelationMapping() {
      $cancelation_mapping = Tools::jsonDecode(Configuration::get('ZEROSIX_CANCELATION_MAPPING'));
      if (!is_array($cancelation_mapping)) {
        $cancelation_mapping = [];
      }
      return $cancelation_mapping;
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $res = array(
          'ZEROSIX_URL_LOYALTY' => Configuration::get('ZEROSIX_URL_LOYALTY'),
          'ZEROSIX_URL_TERMS' => Configuration::get('ZEROSIX_URL_TERMS'),
          'ZEROSIX_URL_DASHBOARD' => Configuration::get('ZEROSIX_URL_DASHBOARD'),
          'ZEROSIX_SPONSORSHIP' => Configuration::get('ZEROSIX_SPONSORSHIP'),
          'ZEROSIX_URL_SPONSORSHIP' => Configuration::get('ZEROSIX_URL_SPONSORSHIP'),
          'ZEROSIX_PHONE_CODE' => Configuration::get('ZEROSIX_PHONE_CODE')
        );

        $shops = Shop::getShops();
        foreach($shops as $shop) {
          $res['ZEROSIX_API_KEY_'.$shop['id_shop']] = Configuration::get('ZEROSIX_API_KEY_'.$shop['id_shop']);
          if($this->is_pos) {
            foreach($this->pos as $pos_name) {
              $res['ZEROSIX_API_KEY_'.$pos_name.'_'.$shop['id_shop']] = Configuration::get('ZEROSIX_API_KEY_'.$pos_name.'_'.$shop['id_shop']);
            }
          }
        };

        $cancelation_mapping = $this->getCancelationMapping();

        foreach($cancelation_mapping as $cancelation_id) {
          $res['ZEROSIX_CANCELATION_MAPPING_'.$cancelation_id] = true;
        }

        $validation_mapping = $this->getValidationMapping();
        
        foreach($validation_mapping as $validation_id) {
          $res['ZEROSIX_VALIDATION_MAPPING_'.$validation_id] = true;
        }
        
        return $res;
      }
      
      /**
       * Save form data.
       */
      protected function postProcess()
      {
        $success = true;
        $api_errors = false;
        $errors = "";
        $form_values = $this->getConfigFormValues();

        $cancelation_mapping = [];
        foreach(Tools::getAllValues() as $key => $value) {
          if (strpos($key, "ZEROSIX_CANCELATION_MAPPING", 0) === 0 && $value == true) {
            $cancelation_id= array_slice(explode("_", $key), -1)[0];
            $cancelation_mapping[] = $cancelation_id;
          }
        }

        Configuration::updateValue('ZEROSIX_CANCELATION_MAPPING', Tools::jsonEncode($cancelation_mapping));
       
        $validation_mapping = [];
        foreach(Tools::getAllValues() as $key => $value) {
          if (strpos($key, "ZEROSIX_VALIDATION_MAPPING", 0) === 0 && $value == true) {
            $validation_id= array_slice(explode("_", $key), -1)[0];
            $validation_mapping[] = $validation_id;
          }
        }

        $api_keys = [];
        foreach(Tools::getAllValues() as $key => $value) {
          if(strpos($key, "ZEROSIX_API_KEY_", 0) === 0 && $value != '') {
            array_push($api_keys, $value);
          }
        }

        if (count($api_keys) != count(array_unique($api_keys))) {
          $success = false;
          $api_errors = true;
          $errors .= $this->displayError($this->l('You cannot use the same api key for different stores'));
        }

        if(empty($validation_mapping)) {
          $success = false;
          $errors .= $this->displayError($this->l('Please choose at least one validation method'));
        }
        else {
          Configuration::updateValue('ZEROSIX_VALIDATION_MAPPING', Tools::jsonEncode($validation_mapping));
        }
        foreach (array_keys($form_values) as $key) {
            if($key == "ZEROSIX_URL_LOYALTY" || $key == "ZEROSIX_URL_TERMS" || $key == "ZEROSIX_URL_DASHBOARD")
            {
              if(Validate::isUrl(Tools::getValue($key)))
              {
                Configuration::updateValue($key, Tools::getValue($key));
              }
              else
              {
                $success = false;
                $errors .= $this->displayError($this->l('Please enter an URI for field: ').$key);
              }
            } else if($key == "ZEROSIX_URL_SPONSORSHIP") {
              if(Tools::getValue("ZEROSIX_SPONSORSHIP")) {
                if(Validate::isUrl(Tools::getValue($key)))
                {
                  Configuration::updateValue($key, Tools::getValue($key));
                }
                else {
                  $success = false;
                  $errors .= $this->displayError($this->l('Please enter an URI for field: ').$key);
                }
              }
              else {
                Configuration::updateValue($key, null);
              }
            } 
            else if(strpos($key, "ZEROSIX_API_KEY_", 0) === 0 && $value != '') {
              if(!$api_errors) {
                Configuration::updateValue($key, Tools::getValue($key));
              }
            }else
            {
              Configuration::updateValue($key, Tools::getValue($key));
            }
        }
        return array('success' => $success, 'errors' => $errors);
    }

    public function updateAccount($customer, $zsphone, $zsSponsorCode, $prefered_language_id, $is_pos=false) 
    {
      if(!$this->isAlreadyMapped($zsphone))
      {
        $card     = $this->checkIfCardExists($zsphone);
        
        if($card && !$is_pos)
        {
          if(!$this->isWaitingOtp())
          {
            if($this->sendOtp($zsphone))
            {
              $this->registerOtp($zsphone,(int)$customer->id);
            }
          }
          return;
      
        }else {
          // create card in ZS AND map in prestashop
          $this->createCardZS($zsphone, $zsSponsorCode, $customer, $prefered_language_id);
        }
      }
    }

    public function hookactionObjectCustomerAddAfter($params)
    {
      if(Tools::isSubmit('zs_terms') && Tools::getValue('zs_terms') == "1") {
        $zsphone  = pSQL(Tools::getValue('zsphone'));
        $zsSponsorCode = null;
        $prefered_language_id = null;
        if(Tools::isSubmit('zs_sponsor_code') && Tools::getValue('zs_sponsor_code') != "") {
          $zsSponsorCode = Tools::getValue('zs_sponsor_code');
        }
        if(Tools::getValue('prefered_language_id') != "") {
          $prefered_language_id = Tools::getValue('prefered_language_id');
        }
        if($zsphone != '')
        {
          $customer = (object)$params['object'];
          $this->updateAccount($customer, $zsphone, $zsSponsorCode, $prefered_language_id);
        }
      }
    }

    public function hookactionCustomerAccountAdd($params) {
      $shop_id = (int)$this->context->shop->id;
      $phone = $this->getPhoneNum($params['newCustomer']->id);
      if (!$this->isAlreadyMapped($phone) && $this->isWaitingOtp()) {
        return Tools::redirect($this->context->link->getModuleLink($this->name, "account")."?from_page=".$this->context->controller->php_self);
      }
    }

    /*
    * Remove user from ZS when removing from PS
    */

    public function hookactionObjectCustomerDeleteAfter($params)
    {
      $customer = (object)$params['object'];

      $user_zs  = $this->getUserZS((int)$customer->id);

      if($user_zs)
      {
        $delete = $this->deleteCardFromZS($user_zs);

        if($delete !== false){
          $this->deleteCardFromPS((int)$customer->id);
          $this->deleteVoucherPS((int)$customer->id);
          $this->removeOtp((int)$customer->id);
        }

      }

    }

    /*
    * Remove cart rule in PS.
    */

    public function hookactionObjectCartRuleDeleteAfter($params)
    {
      $cartRule = (object)$params['object'];
      Db::getInstance()->delete('zerosix_map_vouchers', 'id_discount_ps = '.$cartRule->id);
    }



    /*
    * ZS
    */

    public function get_access_key($shop_id) {
      $context = Context::getContext();
      foreach ($this->pos as $pos_name) {
        $pos_from_context = isset($context->{$pos_name});
        if($pos_from_context) {
          return Configuration::get('ZEROSIX_API_KEY_'.$pos_name.'_'.$shop_id);
        }
      }
      return Configuration::get('ZEROSIX_API_KEY_'.$shop_id);
    }

    public function createCardZS($num,$sponsorCode,$customer, $prefered_language_id)
    {
      $num = zerosix_localize_mobile_number($num);
      if($this->isAlreadyMapped($num))
      {
        PrestaShopLogger::addLog("ZEROSIX Card already mapped, user id: ".$customer->id. "try to map card with number $num", 3);
        return false;
      }
      
      $shop_id = (int)$this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();

      $birthDate = $customer->birthday;
      if ($birthDate == "0000-00-00") {
        $birthDate = null;
      }

      $gender = null;
      if(isset($customer->id_gender) && in_array($customer->id_gender, array(1,2))) {
        $gender = $customer->id_gender;
      }
      $data    = array(
        'mobile' => $num,
        'first_name' => $customer->firstname,
        'last_name' => $customer->lastname,
        'email' => $customer->email,
        'create_card' => true,
        'opt_out' => false,
        'birth_date' => $birthDate,
        'sponsor_card_reference' => $sponsorCode,
        'gender' => $gender,
        'prefered_language_id' => $prefered_language_id,
      );
      curl_setopt($request, CURLOPT_URL, URL_API.'/contacts/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_POST, 1);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_POSTFIELDS, $data);

      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      curl_close($request);
      if($status_code !== 201)
      {
        $response_content = json_decode($response);
        if($status_code == 400) {
          if(isset($response_content->mobile)) {
            PrestaShopLogger::addLog("ZEROSIX Error when CREATE card with number: $num  ON ZS API, code status => $status_code", 5);
            if (Tools::isSubmit('submitZSCard')) {
              $error[] = $this->l('Please enter a valid phone number');
              $this->context->smarty->assign('error',$error);
            }
            return false;
          }

          if(isset($response_content->sponsor_card_reference)) {
            PrestaShopLogger::addLog("ZEROSIX Error when CREATE card with sponsor code: $sponsorCode  ON ZS API, code status => $status_code", 5);
            if (Tools::isSubmit('submitZSCard')) {
              $error[] = $this->l('This sponsor code is not valid');
              $this->context->smarty->assign('error', $error);
            }
            return false;
          }
        }
        PrestaShopLogger::addLog("ZEROSIX Error when CREATE card with number: $num  ON ZS API, code status => $status_code", 5);
      }

      $card     = $this->checkIfCardExists($num);
      if($card){
        $id_user_zs = $this->getIdFromZSApiUrl($card->url);
        $id_card_zs = $this->getIdFromZSApiUrl($card->card);

        return $this->mapExistingCard($customer->id, $id_user_zs, $id_card_zs, $shop_id,$num);
      }else{
          $zs_contact_id = $this->getContactIdUrlZS($num);
          if($zs_contact_id != false) {
            $post_fields = array(
              "mobile" => $num,
              "create_card" => "true"
            );
            $request = curl_init();
            curl_setopt($request, CURLOPT_URL, URL_API.'/contacts/' . $zs_contact_id);
            curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($request, CURLOPT_CUSTOMREQUEST, "PUT");
            curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
            curl_setopt($request, CURLOPT_POSTFIELDS, $post_fields);
            $response = curl_exec($request);
            $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
            curl_close($request);
            
          $card = $this->checkIfCardExists($num);
          if ($card) {
              $id_user_zs = $this->getIdFromZSApiUrl($card->url);
              $id_card_zs = $this->getIdFromZSApiUrl($card->card);
          } else {
            return false;
          }
          return $this->mapExistingCard($customer->id, $id_user_zs, $id_card_zs, $shop_id,$num); 
        }
      }
    }

    public function mapExistingCard($id_customer_presta,$id_user_zs,$id_card_zs, $shop_id,$phone_num = null){
      if($phone_num != null)
      {
        $phone_num = zerosix_localize_mobile_number($phone_num);
      }
      if(!$this->isAlreadyMapped($phone_num))
      {
        return Db::getInstance()->insert('zerosix_map_user', array(
          'id_customer_presta' => (int)$id_customer_presta,
          'id_user_zs' => (int)$id_user_zs,
          'id_card_zs' => (int)$id_card_zs,
          'phone_num' => $phone_num,
          'id_shop' => $shop_id
        ));
      }else {
          PrestaShopLogger::addLog("ZEROSIX Card already mapped with phone number $phone_num with customer ID: $id_customer_presta", 3);
          return false;
      }
    }

    public function getContactIdUrlZS($mobile) {
      $mobile        = zerosix_localize_mobile_number($mobile);
      $shop_id = (int)$this->context->shop->id;
      $access_key    = Configuration::get('ZEROSIX_API_KEY_'.$shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION,
        'Content-Type: application/json');
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API."/contacts?q=" . urlencode($mobile));
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_FOLLOWLOCATION, true);
      $response = json_decode(curl_exec($request));
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      if($status_code == 404) {
        return 404;
      }

      $contact_id = $this->getIdFromZSApiUrl($response->results[0]->url);
      return $contact_id;
    }

    public function getCardInfoZS($card_num)
    {
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API."/cards/$card_num/");
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);

      if($status_code !== 200) {
        if ($status_code == 404) {
            $id_customer = ($this->context->controller->controller_type == "admin") ? (int)Tools::getValue('id_customer') : (int)$this->context->customer->id;
            $this->deleteCardFromPS($id_customer);
            $this->deleteVoucherPS($id_customer);
            $this->removeOtp($id_customer);
        }
        PrestaShopLogger::addLog("ZEROSIX Error when get card $card_num ON ZS API, code status => $status_code", 3);
        return false;
      }
      curl_close($request);
      $response_content = json_decode($response);
      if(isset($response_content)){
        return $response_content;
      }else{
        return false;
      }
    }

    public function registerOtp($num, $id_customer) {
      $num          = zerosix_localize_mobile_number($num);
      $data         = array('id_user' => $id_customer, 'number' => $num );
      $this->removeOtp($id_customer);
      return Db::getInstance()->insert('zerosix_waiting_otp', $data);
    }

    public function removeOtp($id_customer) {
      return Db::getInstance()->delete('zerosix_waiting_otp', 'id_user = '.pSQL($id_customer));
    }

    public function sendOtp($num)
    {
      $num        = zerosix_localize_mobile_number($num);
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION,
        'Content-Type: application/json');
      $data    = array(
        'identifier' => $num
      );
      $data_string = Tools::jsonEncode($data);
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API.'/otp/send/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_POST, 1);

      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_POSTFIELDS, $data_string);

      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      
      curl_close($request);
      if($status_code !== 200)
      {
          PrestaShopLogger::addLog("ZEROSIX Error when check card with number $num ON ZS API, code status => $status_code", 3);
          return false;
      }
      return true;
    }

    public function isWaitingOtp()
    {
      $id_customer = (int)$this->context->customer->id;
      $res =  Db::getInstance()->getRow('SELECT * FROM '._DB_PREFIX_.'zerosix_waiting_otp WHERE id_user = '.pSQL($id_customer));
      if(is_array($res) && count($res) > 0 || $res) 
      {
        return $res;
      }else{
        return false;
      }
    }

    public function checkOtp($num,$token)
    {
      $num        = zerosix_localize_mobile_number($num);
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION,
        'Content-Type: application/json');
      $data    = array(
        'identifier' => $num,
        'token' => $token
      );
      $data_string = Tools::jsonEncode($data);
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API.'/otp/check/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_POST, 1);

      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_POSTFIELDS, $data_string);

      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      curl_close($request);
      
      if($status_code !== 200)
      {
          PrestaShopLogger::addLog("ZEROSIX Error when check card with number $num ON ZS API, code status => $status_code", 3);
          return false;
      }
      return Tools::jsonDecode($response);
    }



    public function checkIfCardExists($num)
    {
      $num        = zerosix_localize_mobile_number($num);
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API.'/contacts/?q='.urlencode($num));
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      if($status_code !== 200)
      {
        PrestaShopLogger::addLog("ZEROSIX Error when check card with number $num ON ZS API, code status => $status_code", 3);
        return false;
      }
      curl_close($request);
      $response_content = json_decode($response);
      if(isset($response_content->results) && isset($response_content->results[0]->card)){
        return $response_content->results[0];
      }else{
        return false;
      }
    }

    public function isAlreadyMapped($num)
    {
      $shop_id = (int)$this->context->shop->id;
      $formated = zerosix_localize_mobile_number($num);
      $res = Db::getInstance()->getValue("SELECT id_customer_presta FROM "._DB_PREFIX_."zerosix_map_user WHERE (phone_num = '".$num."' OR phone_num = '".$formated."') AND id_shop = ".$shop_id."");
      if($res != '') {
        $id_ps_customer = (bool)Db::getInstance()->getValue("SELECT id_customer FROM "._DB_PREFIX_."customer WHERE id_customer = ".(int)$res);
        if(!$id_ps_customer) {
          $this->deleteCardFromPS((int)$res);
          return false;
        }
        return true;
      }
      return false;
    }

    /*
    * get ZS card ref by id user in prestashop
    */
    public function getCard($id_customer_presta)
    {
      return Db::getInstance()->getValue("SELECT id_card_zs FROM "._DB_PREFIX_."zerosix_map_user WHERE id_customer_presta = ".(int)$id_customer_presta);
    }

    /*
    * get ZS user ref by id user in prestashop
    */
    public function getUserZS($id_customer_presta)
    {
      return Db::getInstance()->getValue("SELECT id_user_zs FROM "._DB_PREFIX_."zerosix_map_user WHERE id_customer_presta = $id_customer_presta");
    }

    /*
    * getphone user
    */
    public function getPhoneNum($id_customer_presta)
    {
      return Db::getInstance()->getValue("SELECT phone_num FROM "._DB_PREFIX_."zerosix_map_user WHERE id_customer_presta = $id_customer_presta");
    }



    /*
    * WithDRAW CASHBACK ZS
    */

    public function withDrawCashback($card,$amount)
    {
      $shop_id = (int)$this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);

      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      $data    = array(
        'card' => URL_API."/cards/$card/",
        'cashback_withdraw_value' => $amount,
        'price' => $amount
      );
      curl_setopt($request, CURLOPT_URL, URL_API.'/visits/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_POST, 1);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_POSTFIELDS, $data);

      $response = curl_exec($request);
      $response = json_decode($response);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      curl_close($request);
      if($status_code !== 201)
      {
        PrestaShopLogger::addLog("ZEROSIX Error when POST VISIT WITHDRAW CASHBACK, ZS CARD: $card , Amount: $amount code status => $status_code", 3);
        return false;
      }else{
        return $response;
      }
    }

    public function mappingWithdraw($id_user_ps,$amount,$visit_zs)
    {
      $cart_rule  = new CartRule;
      $cart_rule->id_customer = $id_user_ps;
      $cart_rule->quantity = 1;
      $cart_rule->quantity_per_user = 1;
      $cart_rule->minimum_amount_currency = 1;
      $cart_rule->reduction_amount = (float)$amount;
      $cart_rule->reduction_tax = true;
      $cart_rule->reduction_currency = 1;
      $cart_rule->active = true;
      $cart_rule->highlight = true;

      $cart_rule->date_from = date('Y-m-d');
      $cart_rule->date_to = date('Y-m-d', strtotime('+1 months'));
      $langs = Language::getLanguages();
      $random_name = "ZS_".uniqid();
      foreach($langs as $l)
      {
        $cart_rule->name[$l['id_lang']] = $this->l('Cashback discount').' '.$amount;
      }
      $cart_rule->code = $random_name;
      if($cart_rule->save())
      {
        return Db::getInstance()->insert('zerosix_map_vouchers',
          array(
              'id_discount_ps' => $cart_rule->id,
              'id_user_ps' => $id_user_ps,
              'id_voucher_zs' => $visit_zs,
              'type' => 'cashback'
          )
        );
      }
    }

    public function getShopWithLoyaltyEnabled($shops) {
      $shop_with_loyalty = array();
      foreach($shops as $shop) {
        $access_key = Configuration::get("ZEROSIX_API_KEY_".$shop['id_shop']);
        if($access_key && $access_key != '') {
          array_push($shop_with_loyalty, $shop);
        }
        else {
          foreach($this->pos as $pos_name) {
            $access_key = Configuration::get("ZEROSIX_API_KEY_".$pos_name."_".$shop['id_shop']);
            if($access_key && $access_key != '') {
              array_push($shop_with_loyalty, $shop);
            }
          }
        }
      }
      return $shop_with_loyalty;
    }

    public function mapVoucher($voucherZS, $customer_id)
    {
      $voucherIDZS = $this->getVoucherCode($voucherZS);
      $shops = Shop::getShops();

      if($this->isVoucherAlreadyMapped($voucherIDZS, $customer_id)){
        return false;
      }

      $cart_rule  = new CartRule;
      $cart_rule->id_customer = $customer_id;
      $cart_rule->quantity = 1;
      $cart_rule->quantity_per_user = 1;
      $cart_rule->minimum_amount_currency = Configuration::get('PS_CURRENCY_DEFAULT');
      if($voucherZS->unit == "%")
      {
        $cart_rule->reduction_percent = (float)$voucherZS->value;
      } else  {
        $cart_rule->reduction_amount = (float)$voucherZS->value;
      } 

      if($voucherZS->minimum_amount) {
        $cart_rule->minimum_amount = (float)$voucherZS->minimum_amount;
        $cart_rule->minimum_amount_tax = 1;
        $cart_rule->minimum_amount_shipping = 0;
      }

      $cart_rule->reduction_tax = true;
      $cart_rule->reduction_currency = Configuration::get('PS_CURRENCY_DEFAULT');
      $cart_rule->active = true;
      $cart_rule->highlight = true;

      $cart_rule->date_from = date('Y-m-d');

      if(count($shops) > 1) {
        $cart_rule->shop_restriction=1;
      }

      if(!isset($voucherZS->expiration_date) || $voucherZS->expiration_date == null)
      {
        $cart_rule->date_to = date('Y-m-d', strtotime('+2 months'));
      }else{
        // need fix !
        $date = $voucherZS->expiration_date;
        $real_date = new DateTime($date);
        $cart_rule->date_to = $real_date->format('Y-m-d');
      }
      $langs = Language::getLanguages();
      $random_name = $voucherZS->code;
      foreach($langs as $l)
      {
        $cart_rule->name[$l['id_lang']] = $voucherZS->name;
      }
      $cart_rule->code = $random_name;

      if($cart_rule->save())
      {
        if(count($shops) > 1) {
          $shop_with_access = $this->getShopWithLoyaltyEnabled($shops);
          foreach($shop_with_access as $shop) {
            Db::getInstance()->insert('cart_rule_shop', 
            array(
              'id_cart_rule' => $cart_rule->id, 
              'id_shop'=> (int)$shop['id_shop']));
          }
        }

        return Db::getInstance()->insert('zerosix_map_vouchers',
          array(
              'id_discount_ps' => $cart_rule->id,
              'id_user_ps' => $customer_id,
              'id_voucher_zs' => $voucherIDZS,
              'type' => 'voucher'
          )
        );
      }
    }

    public function isVoucherAlreadyMapped($voucherIDZS,$customer_id) {
      return (bool)Db::getInstance()->getValue("SELECT id_discount_ps FROM "._DB_PREFIX_."zerosix_map_vouchers WHERE id_voucher_zs = '$voucherIDZS' AND id_user_ps = $customer_id ");
    }


    private function buildVisitDetails($id_order_ps) {
      $order = new Order($id_order_ps);

      $discount_ratio = ($order->total_products_wt - $order->total_discounts) / $order->total_products_wt;
            
      $detail_items = array();
      $ps_details = OrderDetail::getList($id_order_ps);
      foreach ( $ps_details as $item ) {
        $product = new Product($item["product_id"], false);
        array_push($detail_items, array(
          'quantity' => $item['product_quantity'],
          'total_price' => $item["total_price_tax_incl"] * $discount_ratio,
          'name' => $item["product_name"],
          'unit_price' => $item["unit_price_tax_incl"],
          'description' => $product->description[1],
          'reference' => $item["product_reference"],
          'category' => $product->id_category_default
          )
        );
      }
      return array("items" => $detail_items);
    }

    private function sendVisitZS($card, $price, $id_order_ps, $used_vouchers)
    {
      $order = new Order($id_order_ps);
      $access_key = $this->get_access_key($order->id_shop);
      if($access_key && $access_key != '') {
        $headers = array(
          'Authorization: Token '.$access_key,
          'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION,
          'Content-Type: application/json',
        );
        $request = curl_init();
        $data    = array(
          'card' => URL_API."/cards/".$card."/",
          'price' => $price,
          'comment' => 'Commande en ligne num: '.$id_order_ps,
          'details' => $this->buildVisitDetails($id_order_ps),
          'used_vouchers' => $used_vouchers,
          'reference' => $id_order_ps
        );
        curl_setopt($request, CURLOPT_URL, URL_API.'/visits/');
        curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($request, CURLOPT_POST, 1);
        curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($request, CURLOPT_POSTFIELDS, json_encode($data));

        $response = curl_exec($request);
        $response = json_decode($response);
        $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
        curl_close($request);
        if($status_code !== 201)
        {
          PrestaShopLogger::addLog("ZEROSIX Error when send visit ON ZS API, ZS CARD: $card , price: $price code status => $status_code", 3);
          return false;
        }else{
          $id_visit_zs = $this->getIdFromZSApiUrl($response->url);
          if(!$this->mapVisits($id_order_ps,$id_visit_zs))
          {
            PrestaShopLogger::addLog("ZEROSIX Error when MApping visit ON Prestashop, ID VISITS $id_visit_zs, id_order => $id_order_ps", 5);
            return false;
          }
          return $response;
        }
      }
      return false;
    }

    /*
    * Mapping visits in PS
    */

    private function mapVisits($id_order, $id_visit_zs)
    {
     return Db::getInstance()->insert('zerosix_map_visits', array(
              'id_order_ps' => (int)$id_order,
              'id_visit_zs' => (int)$id_visit_zs
          ));
    }

    /*
    * Check if an order is already mapped
    * return id ZS visits
    */

    public function isOrderMapped($id_order_ps)
    {
      $res = Db::getInstance()->getValue("SELECT id_visit_zs FROM "._DB_PREFIX_."zerosix_map_visits WHERE id_order_ps = $id_order_ps");
      if($res != '')
      {
        return true;
      }else {
        return false;
      }
    }

    public function getZsVisitIdFromOrderId($id_order_ps)
    {
      return Db::getInstance()->getValue("SELECT id_visit_zs FROM "._DB_PREFIX_."zerosix_map_visits WHERE id_order_ps = $id_order_ps");
    }

    /*
    * Hook identity customer
    */

    public function hookdisplayCustomerIdentityForm($params)
    {
      $id_customer = (int)$this->context->customer->id;
      $phone_num = $this->getPhoneNum($id_customer);
      $store_infos = $this->getStoreInformations();
      $multi_languages_enabled = $store_infos->options->multi_languages_enabled;
      $available_languages = $store_infos->available_languages;
      $this->context->smarty->assign('zs_phone',$phone_num);
      $this->context->smarty->assign('terms_uri',Configuration::get('ZEROSIX_URL_TERMS'));
      $this->context->smarty->assign('sponsorship_enabled',Configuration::get('ZEROSIX_SPONSORSHIP'));
      $this->context->smarty->assign('about_sponsorship',Configuration::get('ZEROSIX_URL_SPONSORSHIP'));
      $this->context->smarty->assign('multi_languages_enabled', $multi_languages_enabled);
      $this->context->smarty->assign('available_languages', $available_languages);
      if($this->is_ps17)
      {
        $this->context->controller->addJS($this->_path . 'views/js/front17.js');
        return $this->context->smarty->fetch('module:zerosix/views/templates/hooks/customerIdentityPhoneField17.tpl');
      }else{
        return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/customerIdentityPhoneField.tpl');
      }
    }
    /*
    * hook display field form account
    */

    public function hookdisplayCustomerAccountForm($params)
    {
      $shop_id = $this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);

      if($access_key && $access_key != "" ) {
        return $this->hookdisplayCustomerIdentityForm($params);
      }
    }

    private function orderValidation($order, $state, $id_customer) {
      $validation_mapping = $this->getValidationMapping();
      $id_order = $order->id;
      if(in_array($state->id, $validation_mapping) && !$this->isOrderMapped($id_order))
      {
        $card_zs = $this->getCard((int)$id_customer);
        if($card_zs)
        {
          $customer_vouchers_ps = $this->getVouchersPS($id_customer,true,true);
          $used_vouchers = array();

          foreach($order->getCartRules() as $d)
          {
            $id_cart_rule = (int)$d['id_cart_rule'];
            if(isset($customer_vouchers_ps[$id_cart_rule]))
            {
              array_push($used_vouchers, URL_API."/vouchers/".$customer_vouchers_ps[$id_cart_rule]);
            }
          }
          $this->sendVisitZS($card_zs, number_format((float)$order->total_products_wt, 2, '.', '') ,$id_order, $used_vouchers);
          $this->checkAndCreateVouchers($id_customer);
        } 
      } 
    }

    /*
    * hook action on validate order.
    */
    public function hookactionValidateOrder($params)
    {
      $order        = (object)$params['order'];
      $id_state     = (int)$order->current_state;
      $customer     = (object)$params['customer'];
      $id_customer  = (int)$customer->id;
      $id_order     = (int)$order->id;
      $state        = new OrderState($id_state);

      $this->orderValidation($order, $state, $id_customer);
    }

    /*
    * loadCards
    */
    public function loadCards()
    {
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API.'/stores/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      if($status_code !== 200)
      {
        PrestaShopLogger::addLog("ZEROSIX Error when check stores ON ZS API, code status => $status_code", 3);
        return false;
      }
      curl_close($request);
      $response_content = json_decode($response);
      if(isset($response_content->results) && isset($response_content->results[0])){

        return $response_content->results[0];

      }else{
        return false;
      }
    }


    /*
    * Delete card FROM ZS
    */

    private function deleteCardFromZS($id_user_zs)
    {
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API.'/contacts/'.$id_user_zs.'/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_CUSTOMREQUEST, "DELETE");
      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      curl_close($request);
      if($status_code !== 200)
      {
        PrestaShopLogger::addLog("ZEROSIX Error when check stores ON ZS API, code status => $status_code", 3);
        return false;
      }else{
        return true;
      }
    }

    private function deleteVisitFromZS($id_order)
    {
      $order = new Order($id_order);
      $shop_id = (int)$this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      $id_visit_zs = $this->getZsVisitIdFromOrderId($id_order);
      curl_setopt($request, CURLOPT_URL, URL_API.'/visits/'.$id_visit_zs.'/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      curl_setopt($request, CURLOPT_CUSTOMREQUEST, "DELETE");
      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      curl_close($request);
      if($status_code !== 200)
      {
        PrestaShopLogger::addLog("ZEROSIX Error when check stores ON ZS API, code status => $status_code", 3);
        return false;
      }else{
        return true;
      }
    }

    private function deleteCardFromPS($id_customer_presta)
    {
      return Db::getInstance()->delete('zerosix_map_user','id_customer_presta = '.$id_customer_presta);
    }

    private function deleteVoucherPS($id_customer_presta)
    {
      return Db::getInstance()->delete('zerosix_map_vouchers','id_user_ps = '.$id_customer_presta);
    }

    /*
    * Check and CreateVouchers ZS TO PS
    */

    private function getVoucherCode($zs_voucher) {
      $trimmed_url =  trim($zs_voucher->url, '/');
      return array_slice(explode( '/', $trimmed_url), -1)[0];
    }

    public function checkAndCreateVouchers($id_customer=null)
    {
      $onCart = false;
      if ($id_customer == null) {
        $onCart = true;
        $id_customer = $this->context->customer->id;

        if(!$this->context->customer->logged) {
          return;
        } else if (Tools::getIsset('add')) {
          return;
        } 
      }

      $need_reload = false;
      $customer_vouchers_ps = $this->getVouchersPS($id_customer,true);

      $card = $this->getCard($id_customer);
      if(!$card)
        return;

      $results = $this->getCardInfoZS($card);
      if($results)
      {
          foreach($results->available_vouchers as $zs_voucher)
          {
            $voucher_code = $this->getVoucherCode($zs_voucher);
            if(isset($customer_vouchers_ps[$voucher_code])) {
              unset($customer_vouchers_ps[$voucher_code]);
            } else {
              $this->mapVoucher($zs_voucher,$id_customer);
              $need_reload = true;
            }
          }

          if(sizeof($customer_vouchers_ps) > 0)
          {
            foreach($customer_vouchers_ps as $id_voucher_zs => $id_discount_ps)
            {
              $this->destroyVoucherPS($id_discount_ps);
              $need_reload = true;
            }
          }

          if($onCart && $need_reload)
          {
            $url = Tools::getProtocol();
            $url .=  $_SERVER['HTTP_HOST']  . $_SERVER['REQUEST_URI'];
            return Tools::redirect($url);
          }
      }
    }

    public function unmapVoucher($id_voucher_zs)
    {
      return Db::getInstance()->delete('zerosix_map_vouchers', "id_voucher_zs = $id_voucher_zs");
    }

    public function destroyVoucherPS($id_discount_ps)
    {
      $cartRule = new CartRule($id_discount_ps);
      return $cartRule->delete();

    }

    public function getVouchersPS($id_user_ps, $only_ids = false, $reverse_keys = false)
    {

      $query =  Db::getInstance()->executeS("SELECT * FROM "._DB_PREFIX_."zerosix_map_vouchers WHERE id_user_ps = $id_user_ps AND type != 'cashback' ");

      if($only_ids)
      {
        $results = array();
        foreach($query as $row)
        {
          if(!$reverse_keys)
            $results[$row['id_voucher_zs']] = (int)$row['id_discount_ps'];
          else
            $results[(int)$row['id_discount_ps']] = $row['id_voucher_zs'];

        }
        $query = $results;
      }

      return $query;
    }


    /*
    * Update state order
    */

    public function hookactionOrderStatusUpdate($params)
    {
      $state        = (object)$params['newOrderStatus'];
      $id_order     = (int)$params['id_order'];
      $order        = new Order($id_order);
      $id_customer  = (int)$order->id_customer;
      $cancelation_mapping = $this->getCancelationMapping();
      $validation_mapping = $this->getValidationMapping();

      if (in_array($state->id, $cancelation_mapping)) { // canceled state
        $this->deleteVisitFromZS($id_order);
        $this->checkAndCreateVouchers($id_customer);
      }
      else if( in_array($state->id, $validation_mapping)) { //validation state
        $this->orderValidation($order, $state, $id_customer);
      }
    }


    public function getStoreInformations() {
      $shop_id = (int)$this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      $headers = array(
        'Authorization: Token ' . $access_key,
        'User-Agent: Prestashop/'._PS_VERSION_.'/'.ZS_MODULE_VERSION
      );
      $request = curl_init();
      curl_setopt($request, CURLOPT_URL, URL_API.'/store-informations/');
      curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($request, CURLOPT_HTTPHEADER, $headers);
      $response = curl_exec($request);
      $status_code = curl_getinfo($request, CURLINFO_HTTP_CODE);
      if($status_code !== 200)
      {
        PrestaShopLogger::addLog("ZEROSIX Error when check stores ON ZS API, code status => $status_code", 3);
        return false;
      }
      curl_close($request);
      $response_content = json_decode($response);
      return $response_content;
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookdisplayCustomerAccount($params)
    {
      $shop_id = $this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      
      if($access_key && $access_key != "" ) {
        $this->context->smarty->assign('zs_card', $this->getCard((int)$this->context->customer->id));
        $this->context->smarty->assign('waiting_otp', $this->isWaitingOtp());
        if($this->is_ps17)
        {
          return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayCustomerAccount17.tpl');
        }else{
          return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayCustomerAccount.tpl');
        }
      }
    }

    /*
    * display shopping cart footer
    */

    public function hookdisplayShoppingCartFooter($params)
    {
      $shop_id = $this->context->shop->id;
      $context = Context::getContext();
      $access_key = $this->get_access_key($shop_id);
      
      if($access_key && $access_key != "" ) {
        $card = null;
        if($this->context->customer->logged){
          $card = $this->getCard((int)$this->context->customer->id);
        }
        $this->context->smarty->assign('card',$card);
        $this->context->smarty->assign('waiting_otp',$this->isWaitingOtp());
        if($this->is_ps17)
          return $this->context->smarty->fetch('module:zerosix/views/templates/hooks/displayShoppingCartFooter17.tpl');
        else{
          $this->context->smarty->assign('isLogged',$this->context->customer->logged);
          return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayShoppingCartFooter.tpl');
        }
      }
    }

    /*
    * Hook dashboardZoneTwo
    */

    public function hookdashboardZoneTwo($params)
    {
      $token = Configuration::get('_ZEROSIX_TOKEN_');
      $this->context->smarty->assign('local_token',$token);
      $this->context->smarty->assign('module_dir',_MODULE_DIR_.$this->name.'/');
      $this->context->smarty->assign('dashboard_uri', Configuration::get("ZEROSIX_URL_DASHBOARD"));
      return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/dashboardZoneTwo.tpl');
    }

    /*
    * Hook displayMyAccountBlock
    */
    public function hookdisplayMyAccountBlock($params)
    {
      $shop_id = $this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);
      
      if($access_key && $access_key != "" ) {
        $this->context->smarty->assign('zs_card', $this->getCard((int)$this->context->customer->id));
        return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayMyAccountBlock.tpl');
      }
    }

    /*
    * Hook header (front)
    */

    public function hookdisplayHeader()
    {
      $module_ctrl = 'module-'.$this->name.'-account';
      $ctrl = array('order',$module_ctrl,'discount');

      if($this->is_ps17){
        $ctrl = array('cart','order',$module_ctrl,'discount');
      }

      if ($this->context->controller->php_self == null) {
        if($this->context->controller->controller_type == "modulefront" && $this->context->controller->page_name == $module_ctrl) {
          $this->checkAndCreateVouchers();
        }

      } else {
        if(in_array($this->context->controller->php_self,$ctrl))
        {
          $this->checkAndCreateVouchers();
        }
      }

      if($this->is_ps17)
      {
        $this->context->controller->addJS($this->local_path.'views/js/front17.js');

      }else {
        $this->context->controller->addJS($this->local_path.'views/js/front.js');
      }
      $this->context->controller->addCSS($this->local_path.'views/css/front.css');

      $this->context->smarty->assign('url_cashback', $this->_path.'loadcashback.php');
      $this->context->smarty->assign('confirm_cashback', $this->l('Are you sure ?'));
      return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/header.tpl');
    }

    public function hookHeader()
    {
      return $this->hookdisplayHeader();
    }


    /*
    *  Hook displayPayment
    */

    public function hookdisplayPaymentTop($params)
    {
      $card           = $this->getCard((int)$this->context->customer->id);
      $shop_id = $this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);
      
      if($access_key && $access_key != "" ) {
        $zerosix_price = $this->context->cart->getOrderTotal(true, Cart::BOTH);
        $this->context->smarty->assign('zerosix_price',$zerosix_price);
        $this->context->smarty->assign('card_zs',$card);
        if($this->is_ps17)
        {
          return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayPayment17.tpl');
        }else{
          return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayPayment.tpl');
        }
      }
    }

    /*
    * display cashback in 1.7
    */
    public function hookdisplayReassurance()
    {
      return $this->displayCashback();
    }

    public function displayCashback()
    {
      $shop_id = $this->context->shop->id;
      $access_key = $this->get_access_key($shop_id);
      
      if($access_key && $access_key != "" ) {
        if($this->context->customer->logged)
        {
          $id_customer = $this->context->customer->id;
          $card = $this->getCard($id_customer);
          if(!$card)
            return false;

          $zs_results = $this->getCardInfoZS($card);
          if(!$zs_results)
            return false;

          if($zs_results->program_info->program_type !== "CASHBACK")
            return false;

          if($zs_results->points_balance <= 0)
            return false;

          $account_link = $this->context->link->getModuleLink('zerosix','account');
          $this->context->smarty->assign('account_link',$account_link);
          $this->context->smarty->assign('zs_results',$zs_results);
          $this->context->smarty->assign('cashback',$zs_results->points_balance);
          return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayCashback17.tpl');
        }
      }
      return false;
    }

    /*
    *  Hook displayPayment
    */

    public function hookdisplayAdminCustomers($params)
    {

      if($this->is_ps1765) {
        $id_customer = $this->context->customer->id;
      }
      else {
        $id_customer = (int)Tools::getValue('id_customer');
      }

      $zsSponsorCode  = null;
      $card           = $this->getCard($id_customer);
      $zs_phone       = $this->getPhoneNum($id_customer);
      $error          = array();
      $success        = array();
      $alreadyMapped  = false;


      if(Tools::isSubmit('submitZSCard') && !$card && Tools::getValue('zs_phone') != '')
      {
          $shop_id = (int)$this->context->shop->id;
          $zs_phone = pSQL(Tools::getValue('zs_phone'));
            $check = $this->checkIfCardExists($zs_phone);
            if($check){
              $id_user_zs = $this->getIdFromZSApiUrl($check->url);
              $id_card_zs = $this->getIdFromZSApiUrl($check->card);
              $results = $this->mapExistingCard($id_customer, $id_user_zs, $id_card_zs,$shop_id,$zs_phone);
              if(!$results) {
                if($this->isAlreadyMapped($zs_phone)) {
                  $alreadyMapped = true;
                }
              }
            }else{
              $prefered_language_id = Tools::getValue('prefered_language_id');
              $customer = new Customer($id_customer);
              $results = $this->createCardZS($zs_phone,$zsSponsorCode,$customer, $prefered_language_id);
            }
            if(!$results){
              if(!$alreadyMapped)
                $error[] = $this->l('Error when saving data, try again later please.');
              if($alreadyMapped)
              {
                $error[] = $this->l('This card is already mapped with an other customer');
              }
              $this->context->smarty->assign('error',$error);

            }else{
              $success[] = $this->l('The loyalty card has been created');
              $this->context->smarty->assign('success',$success);
              $card     = $this->getCard((int)$id_customer);
            }
      }

      if(Tools::isSubmit('unmapZSCard')) {
        $this->deleteCardFromPS($id_customer);
        
        $this->removeOtp($id_customer);

        $customer_vouchers_ps = $this->getVouchersPS($id_customer,true);        
        foreach($customer_vouchers_ps as $id_voucher_zs => $id_discount_ps) {
          $this->destroyVoucherPS($id_discount_ps);
        }

        $this->deleteVoucherPS($id_customer);

        $url = Tools::getProtocol();
        $url .=  $_SERVER['HTTP_HOST']  . $_SERVER['REQUEST_URI'];
        return Tools::redirect($url);
      }
      
      if($card)
      {
        $zs_results = $this->getCardInfoZS($card);
        if($zs_results)
        {
          $this->context->smarty->assign('zs_results',$zs_results);
        }
        $this->checkAndCreateVouchers($id_customer);
      }
      $this->context->smarty->assign('card',$card);
      $this->context->smarty->assign('zs_phone',$zs_phone);

      if($this->is_ps1765) {
        $id_customer = $this->context->customer->id;
      }
      else {
        $id_customer = (int)Tools::getValue('id_customer');
      }
      $card           = $this->getCard($id_customer);
      $this->context->smarty->assign('card_zs',$card);
      if($card !== null)
      {
        $zs_results = $this->getCardInfoZS($card);
        $this->context->smarty->assign('zs_results',$zs_results);
      }
      $this->context->smarty->assign('is_ps1765',$this->is_ps1765);
      $this->context->smarty->assign('logo_path', _MODULE_DIR_.'/zerosix/logo.png');
      return $this->context->smarty->fetch($this->local_path.'views/templates/hooks/displayAdminCustomers.tpl');
    }

}
